#-------------------------------------------------------------------------------
# Name:        search
#-------------------------------------------------------------------------------
#!/usr/bin/env python






#-------------------------------------------------------------------------------
# imports we need
# include top level directory (needed to import from helper below)
import sys
import os
#-------------------------------------------------------------------------------



#-------------------------------------------------------------------------------
# import the helper classes
from TWLCHelper import *
#-------------------------------------------------------------------------------
















#---------------------------------------------------------------------------
class TSearch(object):

    def __init__(self):
        # create the TWLCHelper helper
        self.helper = TWLCHelper()
        # defaults?
        self.url_template = self.get_DefaultUrlTemplate()
        self.cachedir = ""
        self.cacheminutes = "60"

    def __del__(self):
        pass


    def get_searchenginename(self):
        return "BASE TSEARCH CLASS"

    def get_DefaultUrlTemplate(self):
        return "DEFAULTURLTEMPLATE not set for search class:"+self.get_searchenginename()


    def DoParseCommandline(self,comlineargs):
        # debug
        #print "ATTN:DEBUG in DoParseCommandline with "+comlineargs

        # set usage
        self.helper.set_usage("sample usage: %prog [options] arg")
        #
        self.helper.get_parser().add_option("", "--search", dest="search", help="search terms string")
        self.helper.get_parser().add_option("", "--id", dest="id", help="optional id")
        self.helper.get_parser().add_option("", "--label", dest="label", help="optional label")
        #
        self.helper.get_parser().add_option("", "--urltemplate", dest="urltemplate", help="optional url template")
        self.helper.get_parser().add_option("", "--urlargs", dest="urlargs", help="optional extra url args")
        #
        self.helper.get_parser().add_option("", "--cachedir", dest="cachedir", help="directory where cache files are stored")
        self.helper.get_parser().add_option("", "--cacheminutes", dest="cacheminutes", help="number of minutes cache should last")
        #
        self.helper.get_parser().add_option("", "--pythonexe", dest="pythonexe", help="path to python.exe")
        # commandline options
        self.helper.get_parser().add_option("", "--infile", dest="infile", help="input file (full path)")
        self.helper.get_parser().add_option("", "--outfile", dest="outfile", help="output file (full path), or defaults to stdout")
        # derived specific
        self.ParseDerivedCommandlineArgs()
        # ok parse some args from commandline
        self.helper.ParseCommandlineParameters(comlineargs)
        # open filein and fileout files (or stdio)
        self.helper.OpenIO()

        # debug
        #print "ATTN:DEBUG done DoParseCommandline with label="+self.helper.get_arg("label","",False)


    def ParseDerivedCommandlineArgs(self):
        # derived classes can call this
        pass




    def RunWithCommandlineOptions(self,comlineargs):
        # parse commandline using helper

        self.DoParseCommandline(comlineargs)
        #raise Exception("ATTN: test exception")


        #

        # get output file
        infile=self.helper.get_filein()
        outfile=self.helper.get_fileout()

        # url template
        urltemplate = self.helper.get_arg("urltemplate","",False)
        if urltemplate!="":
            # overide default url template
            self.url_template = urltemplate

        # cache
        self.cachedir = self.helper.get_arg("cachedir",self.cachedir,False)
        self.cacheminutes = self.helper.get_arg("cacheminutes",self.cacheminutes,False)
        self.helper.set_cachesettings(self.cachedir,self.cacheminutes)

        # python exe passed in?
        self.helper.set_pythonexe(self.helper.get_arg("pythonexe","",False))

        # ok for this script, search terms are passed on commandline
        search = self.helper.get_arg("search","",False)

        #ok now search
        if search!="":
            # if search is passed on commandline, its a single search, do that now
            id = self.helper.get_arg("id","",False)
            label = self.helper.get_arg("label","",False)
            self.DoOneSearch(outfile,id,label,search)
        else:
            # search NOT passed on commandline, means we are going to send in one or more || separated lines as input, and do a search on each
            self.DoSearchFromInfile(infile,outfile)

        # somewhere between 4 and 5 we are having a problem

        # done
        return 0



    def DoSearchFromInfile(self,infile,outfile):
        for inline in infile:
            # ok if its a comment, just pass it along
            inline=inline.strip()
            #print "INLINE FROM INFILE: "+inline
            if (self.helper.iscommentline(inline)):
                outfile.write(inline+"\n")
                continue
            # parse it
            parsedlinedict=self.helper.DoParseProcessLine(inline,False)
            id=""
            search=parsedlinedict["search"]
            label=parsedlinedict["label"]
            self.DoOneSearch(outfile,id,label,search)



    def DoSearch(self,id,label,search):
        # this function does the main work -- take a search term
        # return a list of results
        results = []

        #initialize result with engine name and id
        baseresult = {"engine":self.get_searchenginename()}
        if id!="":
            baseresult["id"]=id
        if label!="":
            baseresult["label"]=label

        # ok grab a web page using our search terms
        content = self.SearchWebPageGrabContents(search)

        # and now parse results
        addresults = self.ParseWebPageContentIntoResults(search,content,baseresult)
        # add
        results.extend(addresults)
        # return
        return results


    def DoOneSearch(self,outfile,id,label,search):
        # ok run the search and get results

        results = self.DoSearch(id,label,search)

        # now output results
        outfile.write("\n// SEARCHING ("+self.get_searchenginename()+") FOR: "+label+"\n")

        for result in results:
            outline = self.helper.MakeOutStringFromDict(result)
            outfile.write(outline+"\n")
#            outline_utf8 = outline.encode('UTF-8')
#            outfile.write(outline_utf8+"\n")
            #outfile.write(outline+"\n")



    def SearchWebPageGrabContents(self,search):
        # conduct search return page conents

        # build url
        url = self.BuildSearchUrlFromSearchString(search)

        # ok grab content
        content = self.helper.GrabFromWebWithCache(url)

        # return it
        return content


    def BuildSearchUrlFromSearchString(self,search):
        # do replacements on template
        url = self.helper.DoTemplateStringReplacements(self.url_template, {"searchTerms":self.helper.urlquoteargs(search)} )

        # extra args?
        urlargs = self.helper.get_arg("urlargs","",False)
        if (urlargs!=""):
            url = url + "&"+urlargs

        return url


    def ParseWebPageContentIntoResults(self,search,content,baseresult):
        # parse content of page request into results
        results = []
        return results
#---------------------------------------------------------------------------






