#-------------------------------------------------------------------------------
# WlcHelper class
#-------------------------------------------------------------------------------
#!/usr/bin/env python

# Version 12/27/11 mouser@donationcoder.com


#---------------------------------------------------------------------------
from optparse import OptionParser
#
import sys
import os
import site
#
import re, htmlentitydefs
#
import subprocess
import urllib
import shlex
from subprocess import Popen, PIPE, STDOUT
#---------------------------------------------------------------------------











#---------------------------------------------------------------------------
# from http://stackoverflow.com/questions/2924310/whats-a-good-equivalent-to-pythons-subprocess-check-call-that-returns-the-cont
class Wlc_CalledProcessError(Exception):
    def __init__(self, returncode, cmd, output=None):
        self.returncode = returncode
        self.cmd = cmd
        self.output = output
    def __str__(self):
        return "Command '%s' returned non-zero exit status %d" % (self.cmd, self.returncode)
#---------------------------------------------------------------------------








#---------------------------------------------------------------------------
class TWLCHelper(object):

    # class (static) variable
    static_classvar_DidPrepareEnvironment=False

    DEF_ArgFileArg = "argfile"
    DEF_ResultSeparator = "|||"

    def __init__(self,commandlineusage=None):
        # singleton to do any preparation?
        TWLCHelper.PrepareWlcEnvironment()
        #
        self.Flag_InvokeCacheScriptViaImport=True
        #
        self.args={}
        self.parser = OptionParser(commandlineusage)
        if commandlineusage!=None:
            self.set_usage(commandlineusage)
        #
        self.parser.add_option("", "--argfile", dest="argfile", help="argument file with additional var=val assignments for commandline args (optional)")
        #
        self.curpath = os.path.realpath(__file__)
        self.curpathdir = os.path.dirname(os.path.realpath(__file__))
#        corepathpos = self.curpath.find("/scripts");
#        if corepathpos==-1:
#            corepathpos = self.curpath.find("\\scripts");
#        if corepathpos>0:
#            self.coredir = self.curpath[0:corepathpos]+"/scripts/core"
#        else:
#            self.coredir = ""
        self.options_cachescriptfname="TCacheGrab"
        self.options_cachescriptpath = self.curpathdir + "/"+self.options_cachescriptfname+".py"
        #print "CACHESCRIPT = " + self.options_cachescriptpath
        self.options_cacheminutes = "200"
        self.options_defaultcachedir = "./cache"
        self.options_cachedir = self.options_defaultcachedir
        self.options_pythonexe = ""
        #
        self.filesopen= False
        pass

    def __del__(self):
        self.closedown()






    @staticmethod
    def PrepareWlcEnvironment():
        # this is a class static function that will only ever run once, and will initialize any extra environment stuff (directories,etc.) that are needed
        if (not TWLCHelper.static_classvar_DidPrepareEnvironment):
            # only run once
            TWLCHelper.static_classvar_DidPrepareEnvironment=True
            # OK this will bring in any python modules under the scripts/lib/site-packages
            curpath = os.path.dirname(os.path.realpath(__file__))
            site.addsitedir(curpath+"/lib/site-packages")
            # OK this will bring in our built in python2 mini-install site-packages even if we are running under another python
            parpath = os.path.normpath(os.path.join(curpath, ".."))
            site.addsitedir(parpath+"/Python2/Lib/site-packages")






    def set_usage(self,commandlineusage):
        self.parser.set_usage(commandlineusage)

    def get_parser(self):
        return self.parser;

    def get_filein(self):
        if (not self.filesopen):
            self.OpenIO()
        return self.filein;

    def get_fileout(self):
        if (not self.filesopen):
            self.OpenIO()
        return self.fileout;

    def get_argispresent(self,argname):
        if (argname in self.args):
            return true
        return false

    def get_arg(self,argname,defaultval,flag_fatalerrorifundefined):
        # get an arguement or return default or throw fatal error
        if (argname in self.args):
            return self.args[argname]
        if (flag_fatalerrorifundefined):
            # fatal error
            print "FATAL ERROR: argument " + argname + " not found, but is required."
            self.closedown()
            exit(1)
        return defaultval

    def get_arg_asint(self,argname,defaultval,flag_fatalerrorifundefined):
        return int(self.get_arg(argname,defaultval,flag_fatalerrorifundefined))

    def set_arg(self,argname,argval):
        self.args[argname]=argval


    def set_cachesettings(self,cachedir,cacheminutes):
        # set cache settings
        if (cachedir==""):
            self.options_cachedir=self.options_defaultcachedir
        else:
            self.options_cachedir=cachedir
        self.options_cacheminutes=cacheminutes


    def get_toolerrorvarname(self):
        return "$$$TOOLERROR"


    def set_pythonexe(self,pythonexe):
        # set optional pythonexe
        self.options_pythonexe=pythonexe


    def ParseCommandlineParameters(self,comlineargs):
        # parse commandline parameters, exit after showing usage or usage error
        # get args from optparse
        if (comlineargs!=None):
            (self.comlineoptions, self.comlineargs) = self.parser.parse_args(self.PrepareComlineArgList(comlineargs))
        else:
            (self.comlineoptions, self.comlineargs) = self.parser.parse_args()
        # save in our dict
        for k, v in self.comlineoptions.__dict__.iteritems():
            if v!=None:
                self.args[k]=v
        # ok now we also allow user to provide us with a file that contains additional options
        if TWLCHelper.DEF_ArgFileArg in self.args:
            fname = self.args[TWLCHelper.DEF_ArgFileArg]
            fp = open(fname,"r")
            lines = fp.readlines()
            fp.close()
            for line in lines:
                if (len(line)<2):
                    continue
                if (line[0:2]=="//"):
                    continue
                linepieces = line.split("=",1)
                if (len(linepieces)<2):
                    continue
                self.args[linepieces[0].strip()]=linepieces[1].strip()


    def PrepareComlineArgList(self,comlineargs):
        if (comlineargs==None):
            return []
        if (True and isinstance(comlineargs, str)):
            # split string into list
            #clist=shlex.split(comlineargs)
            #print "ATTN:DEBUG COMMADNLINE ARGS BEFORE PARSING: "+comlineargs
            lexer = shlex.shlex(comlineargs,posix=True)
            #lexer = shlex.shlex(comlineargs,posix=False)
            lexer.quotes = '"'
            lexer.whitespace_split=True
            clist = list(lexer)
            clisto=[]
            # remove doublequotes around args -- hmm im not sure we want to do this(!) only if they arent removed by commandline parser
            for cword in clist:
                wlen = len(cword)
                if False and wlen>=2 and cword[0]=='"' and cword[wlen-1]=='"':
                    cword = cword[1:wlen-1]
                clisto.append(cword)
            #print "ATTN:DEBUG COMMADNLINE ARGS PARSED FROM STRING:"
            #print clisto
            #exit(1)
            return clisto
        else:
            return comlineargs


    def OpenIO(self):
        # open input and output files
        if (self.filesopen):
            return
        self.filein = self.OpenInputFileOrStdin()
        self.fileout = self.OpenOutputFileOrStdout()


    def OpenInputFileOrStdin(self):
        # open an input file if one passed, or use stdin if not
        fname = self.get_arg('infile','',False)
        if (fname == ''):
            #stdin
            ifile = sys.stdin
        else:
            #open file
            ifile = open(fname,'r')
        return ifile


    def OpenOutputFileOrStdout(self):
        # open an output file if one passed, or use stdout if not
        fname = self.get_arg('outfile','',False)
        if (fname == ''):
            #stdin
            ofile = sys.stdout
        else:
            #open file
            ofile = open(fname,'w')
        return ofile


    def iscommentline(self,line):
        if (len(line)>=2) and (line[0:2]=="//"):
            return True
        if (len(line)>=1) and (line[0:1]=="#"):
            return True
        return False


    def UNUSED_BuildLineListFromInput(self,flag_writecomments):
        # open filein and fileout files (or stdio)
        self.OpenIO()
        # ok do the main work, read from filein, write to fileout
        outlines=[]
        for inline in self.filein:
            # ok if its a comment, just pass it along
            inline=inline.strip()
            if (self.iscommentline(inline)) or inline=="":
                if flag_writecomments:
                    self.fileout.write(inline+"\n")
                continue
            outlines.append(inline)
        return outlines



    def ReadLineListFromInput(self,filein):
        # ok do the main work, read from filein, write to fileout
        outlines=[]
        for inline in filein:
            # ok if its a comment, just pass it along
            inline=inline.strip()
            outlines.append(inline)
        return outlines




    def DoParseProcessLine(self,linestr,flag_CreateFromPlainString):
        # parse line of format TERM=VALUE||TERM=VALUE into a dict
        linedict = {}
        #
        linepieces = linestr.split(TWLCHelper.DEF_ResultSeparator)
        for onepiece in linepieces:
            keyvallist = onepiece.split("=",1)
            if len(keyvallist)>=2:
                linedict[keyvallist[0]]=keyvallist[1]

        if flag_CreateFromPlainString:
            if len(linedict)==0:
                # ok here is for when the line is not already in that format
                pass
            # defaults if not found -- just default to full line
            defaultlabel=linestr
            defaultsearch=linestr
            # ok now add label and search defaults
            if "label" not in linedict:
                linedict["label"]=defaultlabel
            if "search" not in linedict:
                linedict["search"]=defaultsearch

        return linedict


    def WriteProcessOutputLine(self,fileout,linedict):
        # if its just a comment then out comment
        if "commentline" in linedict:
            outline=linedict["commentline"]
        else:
            # ok generate output line
            outline = self.MakeOutStringFromDict(linedict)
        # write it
        #fileout.write(outline.encode('utf-8'))
        fileout.write(outline+"\n")





































    # IT'S A MIRACLE THIS WORKS!!! WHAT A PAIN IT WAS THROWING RANDOM FUNCTION CALLS AT THE WALL TILL I FOUND ONE THAT DIDNT KILL THE PYTHON INTERPRETER.  THANK YOU UNICODE.
    # THIS IS THE ISO-8859-1 WINDOWS VERSION; we should maybe add another that outputs unicode
    def MakeOutStringFromDict(self,linedict):
        outline=u""
        for k,v in linedict.iteritems():
            if len(outline)>0:
                outline+=TWLCHelper.DEF_ResultSeparator
            outline = outline + str(k) +"="
            #
            oneline=u""
            try:
                oneline += v
            except:
                oneline += v.decode('iso-8859-1')
            oneline = self.SafeLineValue(oneline)
            outline += oneline

        uoutline = outline.encode('latin-1','ignore')
        return uoutline




    def SafeLineValue(self,str):
        # make sure str doesnt contain our separator AND doesnt END in |
        str = str.replace(TWLCHelper.DEF_ResultSeparator, "_|_")
        slen = len(str)
        while (slen>0) and (str[slen-1]=='|'):
            str = str[0:slen-1]
            slen = slen - 1
        return str









    def GrabFromWebWithCache(self,url):
        # ok grab a web page by shelling to our grabcache script
        content=""

        if (self.Flag_InvokeCacheScriptViaImport):

            if (True):
                outfilename = self.options_cachedir+"/lastrequest.txt"
                if (os.path.isfile(outfilename)):
                    os.remove(outfilename)
                args = ["--url",url,"--cacheminutes",self.options_cacheminutes,"--cachedir",self.options_cachedir,"--outfile",outfilename]
                argline=self.BuildArgLineFromArgList(args)
                retcode=WlcGlobal_InvokeLocalPythonScriptViaMainImport(self.options_cachescriptfname,argline)
                if (retcode==0):
                    # success, grab output from file
                    infile = open(outfilename,"r")
                    content = infile.read()
            else:
                # standard, get output from console redirect
                args = ["--url",url,"--cacheminutes",self.options_cacheminutes,"--cachedir",self.options_cachedir]
                argline=self.BuildArgLineFromArgList(args)
                # ATTN: unfinished just run and crab console output as content
                (retcode,content)=WlcGlobal_InvokeLocalPythonScriptViaMainImport_CaptureOutput(self.options_cachescriptfname,argline)
            return content
        #
        elif (self.options_pythonexe!=""):
            pythonexearg = self.options_pythonexe
            #pythonexearg = BuildArgLineFromArgList ([self.options_pythonexe])
            args = [pythonexearg,self.options_cachescriptpath,"--url",url,"--cacheminutes",self.options_cacheminutes,"--cachedir",self.options_cachedir]
        else:
            args = [self.options_cachescriptpath,"--url",url,"--cacheminutes",self.options_cacheminutes,"--cachedir",self.options_cachedir]
        argline=self.BuildArgLineFromArgList(args)
        #print "SHELLING: "
        #print args
        #print "SHELLING LINE: "
        #print argline
        #exit(1)
        #try:

        content = self.Wlc_subprocess_check_output_ShellTrue(argline)
        return content




    def FixupRelativeDirectories(self,dir,fullpathtofile):
        # given fullpathtofile, fixup '\.' stuff in dir
        # first get parent DIRECTORY of fullpathtofile
        #print "FIXUP OF: "+dir+" with path "+fullpathtofile
        dirpath = os.path.dirname(fullpathtofile)
        # now replace '.\' stuff with dirpath
        dir = dir.replace("\\.\\", dirpath)
        dir = dir.replace("/./", dirpath)
        if (len(dir)>=2) and ((dir[0:2]=='./') or (dir[0:2]==".\\")):
            dir = dirpath + "/" + dir[2:]
        #print "FIXED TO: "+dir
        return dir


    def urlquoteargs(self,urlargs):
        # quote a url param
        urlargs = urllib.quote_plus(urlargs)
        return urlargs

    def urlnospaces(self,url):
        # quote a url param
        url = url.replace(" ","+")
        return url

    def urlquoteconservatively(self,url):
        # quote a url param
        safechars = "%/:=&?~#+!$,;'@()*[]"
        url = urllib.quote_plus(url,safechars)
        return url


    def LoadAndParseAllResults(self,infile):
        # load all lines from a file, each one is a || separated result
        results = []
        keyindices = []
        #print "IN LoadAndParseAllResults."
        #return results
        for inline in infile:
            # ok if its a comment, just pass it along
            inline=inline.strip()

            if (self.iscommentline(inline)) or (inline==""):
                continue

            # ok parse it and add it
            result = self.DoParseProcessLine(inline,False)
            # ok now we put results in a list organized by searchlabel+id
            index=self.ComputeKeyForResult(result)
            if index not in keyindices:
                keyindices.append(index)
            indexnum = keyindices.index(index)
            if indexnum >= len(results):
                results.append([])
            results[indexnum].append(result)
        #print keyindices
        return results


    def ComputeKeyForResult(self,result):
        # compute str list index key based on id and label
        if "id" in result:
            idval="id"+result["id"]
        else:
            idval=""
        if "label" in result:
            labelval="lbl"+result["label"]
        else:
            labelval=""
        return idval+"_"+labelval


    def RunShellChain(self,shellchain,contents,flag_redirectstartingstdin):
        # shellchain is a list of commandline lines to shell
        for argline in shellchain:
            #print "SHELLING: "+argline
            if flag_redirectstartingstdin:
                # redirect output to input
                p = Popen(argline, shell=True, stdout=PIPE, stdin=PIPE, stderr=STDOUT)
                contents = p.communicate(input=contents)[0]
            else:
                contents = self.Wlc_subprocess_check_output_ShellTrue(argline)
            flag_redirectstartingstdin = True
            #print "OUTPUT: "+contents
        return contents

    def ComputePythonExe(self):
        pythonexepath = sys.executable
        return pythonexepath


    def WriteOutputToFile(self,output,outfile):
        outlines = output.splitlines()
        for line in outlines:
            outfile.write(line+"\n")


    def UnEscapeHtmlText(self,text):
        # from http://effbot.org/zone/re-sub.htm#unescape-html
        def fixup(m):
            text = m.group(0)
            if text[:2] == "&#":
                # character reference
                try:
                    if text[:3] == "&#x":
                        return unichr(int(text[3:-1], 16))
                    else:
                        return unichr(int(text[2:-1]))
                except ValueError:
                    pass
            else:
                # named entity
                try:
                    text = unichr(htmlentitydefs.name2codepoint[text[1:-1]])
                except KeyError:
                    pass
            return text # leave as is
        return re.sub("&#?\w+;", fixup, text)


    def UnEscapeUrl(self,text):
        # from http://stackoverflow.com/questions/3563126/url-encoding-decoding-with-python
        text = urllib.unquote(text)
        return text

    def CleanHtmlExcerptLine(self,text):
        text=text.replace("\r\n","<br />")
        text=text.replace("\r","<br />")
        text=text.replace("\n","<br />")
        return text

    def DoTemplateStringReplacements(self, str, replacedict ):
        for k, v in replacedict.iteritems():
            str=str.replace("{"+k+"}",v)
        return str






    def Handle_Match_ParseDerivedCommandlineArgs(self):
        self.get_parser().add_option("", "--match", dest="match", help="regex that much match lines if line is to be processed")

    def Handle_Match_DoCommandlineOptionsProcess(self,arglist):
        regexflags=re.IGNORECASE
        arglist["match"]=self.get_arg("match","",False)
        arglist["_compiledmatch"]=self.BuildRegexObjectOrNone(arglist["match"],regexflags)


    def Handle_Target_ParseDerivedCommandlineArgs(self):
        self.get_parser().add_option("", "--target", dest="target", help="target of processing (LABEL|SEARCH|URL|LINKTITLE|ALL)")

    def Handle_Target_DoCommandlineOptionsProcess(self,arglist,defaulttarget):
        arglist["target"]=self.get_arg("target",defaulttarget,False)
        arglist["targetlist"]=self.GetTargetDictList(arglist["target"])





    def BuildRegexObjectOrNone(self,matchstr,regexflags):
        if (matchstr==""):
            return None
        return re.compile(matchstr,flags=regexflags)

    def DoMatchTest_OnCompiledMatchArg(self,arglist,linedict):
        matchregexobject=arglist["_compiledmatch"]
        if ("targetlist" in arglist):
            targetlist = arglist["targetlist"]
        else:
            targetlist = []
        return self.DoMatchTest(matchregexobject,targetlist,linedict)


    def DoMatchTest(self,matchregexobject,targetlist,linedict):
        # return true if this line passes regex match test (or if regex match is blank)
        if (matchregexobject == None):
            return True
        result=None
        allowedwords = ["label","search","url","linktitle"]
        for word in allowedwords:
            if (word in linedict):
                result = re.search(matchregexobject, linedict[word])
                if (result!=None):
                    return True
        return False



    def GetTargetDictList(self,targetword):
        # helper function for dealing with targets
        allowedwords = ["label","search","url","linktitle"]
        if (targetword.upper()=="ALL"):
            targetlist=allowedwords
        else:
            words = targetword.split(",")
            # now add and lowercase them, and check for errors
            targetlist = []
            for word in words:
                word = word.lower()
                if (word in allowedwords):
                    targetlist.append(word)
                else:
                    raise Exception("Error in parameter --target: "+word+" is not an allowed field.")
        return targetlist










    def BuildArgLineFromArgList(self,args):
        # new
        #return subprocess.list2cmdline(args)
        # old
        argline=""
        argindex=0
        for arg in args:
            if (False and argindex==0):
                argline +='"'+arg+'" '
            elif (self.ShouldDoubleQuoteArg(arg)):
                argline += '"'+arg+'" '
            else:
                argline += arg+' '
            argindex+=1
        return argline.strip()

    def ShouldDoubleQuoteArg(self,arg):
        if ('&' in arg):
            return True
        if ('?' in arg):
            return True
        if (' ' in arg):
            return True
        if ('"' in arg):
            return True
        return False


    def Wlc_subprocess_check_output_ShellTrue(self,argstuff):
        #print "ASKING TO RUN: "
        #print argstuff
        #exit(1)
        if ( (False) or (sys.version_info < (2, 7)) ):
            # works on python 2.x before 2.7
            return self.Wlc_check_output(argstuff,shell=True)
        # python 2.7:
        # ATTN: try bufsize=-1?
        return subprocess.check_output(argstuff,shell=True)



    # from http://stackoverflow.com/questions/2924310/whats-a-good-equivalent-to-pythons-subprocess-check-call-that-returns-the-cont
    def Wlc_check_output(self,*popenargs, **kwargs):

        # overwrite CalledProcessError due to `output` keyword might be not available
        subprocess.CalledProcessError = Wlc_CalledProcessError

        if 'stdout' in kwargs:
            raise ValueError('stdout argument not allowed, it will be overridden.')
        process = subprocess.Popen(stdout=subprocess.PIPE, *popenargs, **kwargs)
        output, unused_err = process.communicate()
        retcode = process.poll()
        if retcode:
            cmd = kwargs.get("args")
            if cmd is None:
                cmd = popenargs[0]
            if (unused_err!=None):
                # test -- this doesnt work, so we try adding to output
                print unused_ere
                #if (output!=""):
                #    output+="\n"
                #output += unused_err
            #
            raise subprocess.CalledProcessError(retcode, cmd, output=output)
        return output


    def closedown(self):
        # close files, etc
        # ATTN: unfinished
        if (hasattr(self,"filein")):
            self.filein.close()
        if (hasattr(self,"fileout")):
            self.fileout.close()
        pass
#---------------------------------------------------------------------------


























#---------------------------------------------------------------------------
# this is at end because circular imports cause python to get sick and ill and insane
from wlcembedder import *
#---------------------------------------------------------------------------














#---------------------------------------------------------------------------
def main(comlineargs=None):
    pass

if __name__ == '__main__':
    main()
#---------------------------------------------------------------------------
