#!/usr/bin/env python

'''
WikiLinks Extension for Python-Markdown
======================================

Tries to do http://en.wikipedia.org/wiki/Help:URL#Linking_to_URLs
for absolute links

    If the URL is simply included in the wikitext, without square brackets, then it appears as written, and a hyperlink is created: http://meta.wikimedia.org/ produces http://meta.wikimedia.org/
    If the URL is placed in square brackets, then a hyperlink is created with a serial number as its label: [http://meta.wikimedia.org/] gives [1]
    If the URL is placed in square brackets followed by a space and some text, then that text is used as the label for the hyperlink: [http://meta.wikimedia.org/ Wikimedia] gives Wikimedia



This is an ugly kludge by mouser@donationcoder of the wikilinks extension by waylan limberg for relative links.. i dont understand why no one made a proper absolute url wikimedia format extension yet..

By [Waylan Limberg](http://achinghead.com/).

License: [BSD](http://www.opensource.org/licenses/bsd-license.php)

Dependencies:
* [Python 2.3+](http://python.org)
* [Markdown 2.0+](http://www.freewisdom.org/projects/python-markdown/)
'''

import markdown
import re

def build_url(label, base, end):
    """ Build a url from the label, a base, and an end. """
    #clean_label = re.sub(r'([ ]+_)|(_[ ]+)|([ ]+)', '_', label)
    clean_label = label
    return '%s%s%s'% (base, clean_label, end)


class WikiLinkExtension(markdown.Extension):
    def __init__(self, configs):
        # set extension defaults
        self.config = {
                        'base_url' : ['', 'String to append to beginning or URL.'],
                        'end_url' : ['', 'String to append to end of URL.'],
                        'html_class' : ['wikilink', 'CSS hook. Leave blank for none.'],
                        'build_url' : [build_url, 'Callable formats URL from label.'],
        }

        # Override defaults with user settings
        for key, value in configs :
            self.setConfig(key, value)

    def extendMarkdown(self, md, md_globals):
        self.md = md

        # append to end of inline patterns
        WIKILINK_RE = r'\[([^\[\]\s]+)\s*([^\[\]]*)\]'
        wikilinkPattern = WikiLinks(WIKILINK_RE, self.config)
        wikilinkPattern.md = md
        md.inlinePatterns.add('wikilink', wikilinkPattern, "<not_strong")


class WikiLinks(markdown.inlinepatterns.Pattern):
    def __init__(self, pattern, config):
        markdown.inlinepatterns.Pattern.__init__(self, pattern)
        self.config = config

    def handleMatch(self, m):
        if m.group(2).strip():
            base_url, end_url, html_class = self._getMeta()
            label = m.group(2).strip()
            url = self.config['build_url'][0](label, base_url, end_url)
            a = markdown.etree.Element('a')
            if m.group(3).strip()!="":
                label =  m.group(3)
            a.text = label
            a.set('href', url)
            if html_class:
                a.set('class', html_class)
        else:
            a = ''
        return a

    def _getMeta(self):
        """ Return meta data or config data. """
        base_url = self.config['base_url'][0]
        end_url = self.config['end_url'][0]
        html_class = self.config['html_class'][0]
        if hasattr(self.md, 'Meta'):
            if self.md.Meta.has_key('wiki_base_url'):
                base_url = self.md.Meta['wiki_base_url'][0]
            if self.md.Meta.has_key('wiki_end_url'):
                end_url = self.md.Meta['wiki_end_url'][0]
            if self.md.Meta.has_key('wiki_html_class'):
                html_class = self.md.Meta['wiki_html_class'][0]
        return base_url, end_url, html_class


def makeExtension(configs=None) :
    return WikiLinkExtension(configs=configs)


if __name__ == "__main__":
    import doctest
    doctest.testmod()

