#-------------------------------------------------------------------------------
# Name:        process, changecase
#-------------------------------------------------------------------------------
#!/usr/bin/env python






#-------------------------------------------------------------------------------
# imports we need
# include top level directory (needed to import from helper below)
import sys
import os
#-------------------------------------------------------------------------------



#-------------------------------------------------------------------------------
# add parent directory to path
curpath = os.path.dirname(os.path.realpath(__file__))
parpath = os.path.normpath(os.path.join(curpath, ".."))
sys.path.append(parpath)

# titelcase stuff
#from helperlibs.titlecasemodule.titlecase import *
from helperlibs.titlecasemodule import titlecase

# import the parent classes
from TProcess import *
#-------------------------------------------------------------------------------


















#---------------------------------------------------------------------------
class TProcess_ChangeCase(TProcess):

    def __init__(self):
        # parent init
        super(TProcess_ChangeCase, self).__init__()

    def __del__(self):
        pass


    def ParseDerivedCommandlineArgs(self):
        self.helper.get_parser().add_option("", "--case", dest="case", help="case to change to (UPPER|LOWER|SENTENCE|TITLE)")
        self.helper.get_parser().add_option("", "--substring", dest="substring", help="(optional, if specified then only the substring part of the text that matches regex parameter will have its case changed")
        # add target options
        self.helper.Handle_Target_ParseDerivedCommandlineArgs()
        # add match options
        self.helper.Handle_Match_ParseDerivedCommandlineArgs()


    def RunWithCommandlineOptions(self,comlineargs):
        # parse commandline using helper
        self.DoParseCommandline(comlineargs)
        # build extraargs
        extraargs = {}
        extraargs["case"]=self.helper.get_arg("case","",True).upper()
        extraargs["substring"]=self.helper.get_arg("substring","",False)
        extraargs["_compiledsubstring"]=self.helper.BuildRegexObjectOrNone(extraargs["substring"],re.IGNORECASE)
        # preprocess target options
        self.helper.Handle_Target_DoCommandlineOptionsProcess(extraargs,"LABEL")
        # preprocess match options
        self.helper.Handle_Match_DoCommandlineOptionsProcess(extraargs)
        # run it
        retv = self.DoWork(self.helper.get_filein(),self.helper.get_fileout(),extraargs)
        return retv






    # overridden

    def DoProcessLineDict(self,linedict,extraargs):
        # manipulate the linedict in any way one wants

        # match check? we only continue if we match
        if (not self.helper.DoMatchTest_OnCompiledMatchArg(extraargs,linedict)):
            return linedict;

        targetlist = extraargs["targetlist"]

        casestr = extraargs["case"]
        compiledsubstring=extraargs["_compiledsubstring"]

        for tl in targetlist:
            if (tl in linedict):
                linedict[tl]=self.DoCaseReplacementOnField(linedict[tl],casestr,compiledsubstring)

        return linedict



    def DoCaseReplacementOnField(self,fieldstr,casestr,compiledsubstring):
        # substring regex specified?
        if (compiledsubstring!=None):
            # (r'\0',casestr),
            # ok now we do a regex replacement, BUT we pass a function to callback, which according to python docs will receive one parameter the regex object, and we return replacement
            fieldstr=re.sub(compiledsubstring, lambda x: self.Function_CallbackCaseChange(x,casestr), fieldstr)
        else:
            # case change whole string
            fieldstr=self.PerformCaseChangeOnString(fieldstr,casestr)
        return fieldstr




    def Function_CallbackCaseChange(self,matchobj,casestr):
        # return the case change of the regular expression pattern matched on the entire pattern matched (or a group if one exists)
        #
        if (matchobj.lastindex==None):
            # replace entire match, no groups mentioned
            retstr=self.PerformCaseChangeOnString(matchobj.group(0),casestr)
        elif (matchobj.lastindex==1):
            # one group mentioned, replace only that, discard rest
            retstr=self.PerformCaseChangeOnString(matchobj.group(1),casestr)
        elif (matchobj.lastindex==2):
            # two groups mentioned, use first unchanged, replace secndt, discard rest
            retstr=matchobj.group(1)+self.PerformCaseChangeOnString(matchobj.group(2),casestr)
        else:
            # two groups mentioned, use first unchanged, replace secndt, discard rest
            retstr=matchobj.group(1)+self.PerformCaseChangeOnString(matchobj.group(2),casestr)+matchobj.group(3)
        return retstr



    def PerformCaseChangeOnString(self,str,casestr):
        # whole string
        if (casestr=="UPPER"):
            str=str.upper()
        elif (casestr=="LOWER"):
            str=str.lower()
        elif (casestr=="SENTENCE"):
            str=str.capitalize()
        elif (casestr=="TITLE"):
            # built in python titlecase, not good enough
            #label=label.title()
            # use the imported lib instead
            str=titlecase.titlecase(str)
        else:
            # report error?
            # ATTN: unfinished
            str="UNKNOWN CASE CHANGE TYPE '"+casestr+"'."
        return str
#---------------------------------------------------------------------------















































#---------------------------------------------------------------------------
def main(comlineargs=None):
    # make object
    processor = TProcess_ChangeCase()

    #print "ATTN:Hello from changecasemain!!!!!!!!!!!!!!!!!!!!!!!!!!"

    # and ask it to run with commandline options
    return processor.RunWithCommandlineOptions(comlineargs)
#---------------------------------------------------------------------------









#-------------------------------------------------------------------------------
if __name__ == '__main__':
    main()
#-------------------------------------------------------------------------------


